@@ -62,15 +62,13 @@ def _get_query_parameter(url: str, param_name: str) -> Optional[str]:
 def request_new_grant_with_post(
     url: str, data, grant_name: str, client: httpx.Client
 ) -> (str, int):
-    with client:
-        response = client.post(url, data=data)
+    response = client.post(url, data=data)
 
-        if response.is_error:
-            # As described in https://tools.ietf.org/html/rfc6749#section-5.2
-            raise InvalidGrantRequest(response)
-
-        content = response.json()
+    if response.is_error:
+        # As described in https://tools.ietf.org/html/rfc6749#section-5.2
+        raise InvalidGrantRequest(response)
 
+    content = response.json()
     token = content.get(grant_name)
     if not token:
         raise GrantNotProvided(grant_name, content)
@@ -176,9 +174,7 @@ class OAuth2ResourceOwnerPasswordCredentials(httpx.Auth, SupportMultiAuth):
 
         # Time is expressed in seconds
         self.timeout = int(kwargs.pop("timeout", None) or 60)
-        self.client = kwargs.pop("client", None) or httpx.Client()
-        self.client.auth = (self.username, self.password)
-        self.client.timeout = self.timeout
+        self.client = kwargs.pop("client", None)
 
         # As described in https://tools.ietf.org/html/rfc6749#section-4.3.2
         self.data = {
@@ -206,13 +202,24 @@ class OAuth2ResourceOwnerPasswordCredentials(httpx.Auth, SupportMultiAuth):
         yield request
 
     def request_new_token(self) -> tuple:
-        # As described in https://tools.ietf.org/html/rfc6749#section-4.3.3
-        token, expires_in = request_new_grant_with_post(
-            self.token_url, self.data, self.token_field_name, self.client
-        )
+        client = self.client or httpx.Client()
+        self._configure_client(client)
+        try:
+            # As described in https://tools.ietf.org/html/rfc6749#section-4.3.3
+            token, expires_in = request_new_grant_with_post(
+                self.token_url, self.data, self.token_field_name, client
+            )
+        finally:
+            # Close client only if it was created by this module
+            if self.client is None:
+                client.close()
         # Handle both Access and Bearer tokens
         return (self.state, token, expires_in) if expires_in else (self.state, token)
 
+    def _configure_client(self, client: httpx.Client):
+        client.auth = (self.username, self.password)
+        client.timeout = self.timeout
+
 
 class OAuth2ClientCredentials(httpx.Auth, SupportMultiAuth):
     """
@@ -264,9 +271,7 @@ class OAuth2ClientCredentials(httpx.Auth, SupportMultiAuth):
         # Time is expressed in seconds
         self.timeout = int(kwargs.pop("timeout", None) or 60)
 
-        self.client = kwargs.pop("client", None) or httpx.Client()
-        self.client.auth = (self.client_id, self.client_secret)
-        self.client.timeout = self.timeout
+        self.client = kwargs.pop("client", None)
 
         # As described in https://tools.ietf.org/html/rfc6749#section-4.4.2
         self.data = {"grant_type": "client_credentials"}
@@ -290,13 +295,24 @@ class OAuth2ClientCredentials(httpx.Auth, SupportMultiAuth):
         yield request
 
     def request_new_token(self) -> tuple:
-        # As described in https://tools.ietf.org/html/rfc6749#section-4.4.3
-        token, expires_in = request_new_grant_with_post(
-            self.token_url, self.data, self.token_field_name, self.client
-        )
+        client = self.client or httpx.Client()
+        self._configure_client(client)
+        try:
+            # As described in https://tools.ietf.org/html/rfc6749#section-4.4.3
+            token, expires_in = request_new_grant_with_post(
+                self.token_url, self.data, self.token_field_name, client
+            )
+        finally:
+            # Close client only if it was created by this module
+            if self.client is None:
+                client.close()
         # Handle both Access and Bearer tokens
         return (self.state, token, expires_in) if expires_in else (self.state, token)
 
+    def _configure_client(self, client: httpx.Client):
+        client.auth = (self.client_id, self.client_secret)
+        client.timeout = self.timeout
+
 
 class OAuth2AuthorizationCode(httpx.Auth, SupportMultiAuth, BrowserAuth):
     """
@@ -370,9 +386,7 @@ class OAuth2AuthorizationCode(httpx.Auth, SupportMultiAuth, BrowserAuth):
         username = kwargs.pop("username", None)
         password = kwargs.pop("password", None)
         self.auth = (username, password) if username and password else None
-        self.client = kwargs.pop("client", None) or httpx.Client()
-        self.client.auth = self.auth
-        self.client.timeout = self.timeout
+        self.client = kwargs.pop("client", None)
 
         # As described in https://tools.ietf.org/html/rfc6749#section-4.1.2
         code_field_name = kwargs.pop("code_field_name", "code")
@@ -436,13 +450,25 @@ class OAuth2AuthorizationCode(httpx.Auth, SupportMultiAuth, BrowserAuth):
 
         # As described in https://tools.ietf.org/html/rfc6749#section-4.1.3
         self.token_data["code"] = code
-        # As described in https://tools.ietf.org/html/rfc6749#section-4.1.4
-        token, expires_in = request_new_grant_with_post(
-            self.token_url, self.token_data, self.token_field_name, self.client
-        )
+
+        client = self.client or httpx.Client()
+        self._configure_client(client)
+        try:
+            # As described in https://tools.ietf.org/html/rfc6749#section-4.1.4
+            token, expires_in = request_new_grant_with_post(
+                self.token_url, self.token_data, self.token_field_name, client
+            )
+        finally:
+            # Close client only if it was created by this module
+            if self.client is None:
+                client.close()
         # Handle both Access and Bearer tokens
         return (self.state, token, expires_in) if expires_in else (self.state, token)
 
+    def _configure_client(self, client: httpx.Client):
+        client.auth = self.auth
+        client.timeout = self.timeout
+
 
 class OAuth2AuthorizationCodePKCE(httpx.Auth, SupportMultiAuth, BrowserAuth):
     """
@@ -503,8 +529,7 @@ class OAuth2AuthorizationCodePKCE(httpx.Auth, SupportMultiAuth, BrowserAuth):
 
         BrowserAuth.__init__(self, kwargs)
 
-        self.client = kwargs.pop("client", None) or httpx.Client()
-        self.client.timeout = self.timeout
+        self.client = kwargs.pop("client", None)
 
         self.header_name = kwargs.pop("header_name", None) or "Authorization"
         self.header_value = kwargs.pop("header_value", None) or "Bearer {token}"
@@ -590,13 +615,24 @@ class OAuth2AuthorizationCodePKCE(httpx.Auth, SupportMultiAuth, BrowserAuth):
 
         # As described in https://tools.ietf.org/html/rfc6749#section-4.1.3
         self.token_data["code"] = code
-        # As described in https://tools.ietf.org/html/rfc6749#section-4.1.4
-        token, expires_in = request_new_grant_with_post(
-            self.token_url, self.token_data, self.token_field_name, self.client
-        )
+
+        client = self.client or httpx.Client()
+        self._configure_client(client)
+        try:
+            # As described in https://tools.ietf.org/html/rfc6749#section-4.1.4
+            token, expires_in = request_new_grant_with_post(
+                self.token_url, self.token_data, self.token_field_name, client
+            )
+        finally:
+            # Close client only if it was created by this module
+            if self.client is None:
+                client.close()
         # Handle both Access and Bearer tokens
         return (self.state, token, expires_in) if expires_in else (self.state, token)
 
+    def _configure_client(self, client: httpx.Client):
+        client.timeout = self.timeout
+
     @staticmethod
     def generate_code_verifier() -> bytes:
         """
